@@ -1,20 +1,24 @@
 package Log::Log4perl::Appender::Graylog;
 
 # ABSTRACT: Log dispatcher writing to udp Graylog server
-our $VERSION = '1.4'; # VERSION 1.4
-my $VERSION=1.4;
+our $VERSION = '1.5'; # VERSION 1.5
+my $VERSION = 1.5;
 our @ISA = qw(Log::Log4perl::Appender);
 
 use strict;
 use warnings;
 
-use JSON -convert_blessed_universally;
 use Sys::Hostname;
 use Data::UUID;
 use POSIX qw(strftime);
 use IO::Compress::Gzip qw( gzip $GzipError );
 use IO::Socket;
 use Data::DTO::GELF;
+use Carp;
+use Log::GELF::Util qw(
+    :all
+);
+
 
 ##################################################
 # Log dispatcher writing to udp Graylog server
@@ -26,18 +30,33 @@ sub new {
     my $proto  = shift;
     my $class  = ref $proto || $proto;
     my %params = @_;
-
+    
     my $self = {
         name     => "unknown name",
         PeerAddr => "",
         PeerPort => "",
-        Gzip => 1,
+        Proto    => "udp",
+        Gzip     => 1,
+        Chunked  => 0,
         %params,
+        
     };
-
     bless $self, $class;
+    
 }
 
+sub _create_socket {
+    my ( $self, $socket_opts ) = @_;
+
+    require IO::Socket::INET;
+    my $socket = IO::Socket::INET->new(
+        PeerAddr => $socket_opts->{host},
+        PeerPort => $socket_opts->{port},
+        Proto    => $socket_opts->{protocol},
+    ) or die "Cannot create socket: $!";
+
+    return $socket;
+}
 ##################################################
 sub log {
 ##################################################
@@ -54,27 +73,19 @@ sub log {
         "_pid"         => $$,
 
     );
-    # TODO: Use the gzip way, however the specs say it can send plain if there is no new line
-    my $json = JSON->new->utf8->space_after->allow_nonref->convert_blessed;
-    my $j_packet = $json->encode($packet);
 
-    my $socket = IO::Socket::INET->new(
-        PeerAddr => "$self->{'PeerAddr'}",
-        PeerPort => $self->{'PeerPort'},
-        Type     => SOCK_DGRAM,
-        Proto    => 'udp'
-    ) or die "Socket error";
-
-    if($self->{'Gzip'})
-    {
-        my $gzipped_message;
-        gzip \$j_packet =>  \$gzipped_message or die "gzip failed: $GzipError\n";
-        $socket->send( $gzipped_message . "\n" );
-    }else {
-        $socket->send( $j_packet . "\n" );
-    }
-    
 
+    my $msg = validate_message($packet->TO_HASH());
+    my $chunked = parse_size( $self->{Chunked} );
+    $msg = encode($msg);
+    $msg = compress($msg) if $self->{'Gzip'};
+    my $socket = $self->_create_socket(
+        {   'host'     => $self->{'PeerAddr'},
+            'port'     => $self->{'PeerPort'},
+            'protocol' => $self->{'Proto'}
+        }
+    );
+    $socket->send($_) foreach enchunk( $msg, $chunked );
     $socket->close();
 
 }
@@ -93,7 +104,7 @@ Log::Log4perl::Appender::Graylog - Log dispatcher writing to udp Graylog server
 
 =head1 VERSION
 
-version 1.4
+version 1.5
 
 =head1 SYNOPSIS
 
@@ -106,18 +117,36 @@ version 1.4
     );
  
     $appender->log(message => "Log me\n");
+ 
+    or
+    log4perl.appender.SERVER          = Log::Log4perl::Appender::Graylog
+    log4perl.appender.SERVER.layout = NoopLayout
+    log4perl.appender.SERVER.PeerAddr = <ip>
+    log4perl.appender.SERVER.PeerPort = 12201
+    log4perl.appender.SERVER.Gzip    = 1
 
 =head1 DESCRIPTION
 
 This is a simple appender for writing to a graylog server.
-It relies on L<IO::Socket::INET>. This sends in the 1.1
-format. Hoever it does not gzip the message. There are plans
-to use the gzip method later.
+It relies on L<IO::Socket::INET>. L<Log::GELF::Util>. This sends in the 1.1
+format. 
 
 =head1 NAME
 
 Log::Log4perl::Appender::Graylog; - Log to a Graylog server
 
+=head1 CONFIG
+    log4perl.appender.SERVER          = Log::Log4perl::Appender::Graylog
+    log4perl.appender.SERVER.layout = NoopLayout
+    log4perl.appender.SERVER.PeerAddr = <ip>
+    log4perl.appender.SERVER.PeerPort = 12201
+    log4perl.appender.SERVER.Gzip    = 1
+    log4perl.appender.SERVER.Chunked = <0|lan|wan> 
+
+    layout This needs to be NoopLayout as we do not want any special formatting.
+    Gzip Accepts an integer specifying if to compress the message. 
+    Chunked Accepts an integer specifying the chunk size or the special string values lan or wan corresponding to 8154 or 1420 respectively.
+
 =head1 EXAMPLE
 
 Write a server quickly using the IO::Socket:
@@ -169,8 +198,6 @@ Copyright 2017 by Brandon "Dimentox Travanti" Husbands E<lt>xotmid@gmail.comE<gt
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
 
-=cut
-
 =head1 AUTHOR
 
 Brandon "Dimentox Travanti" Husbands <xotmid@gmail.com>
